package POWER::Event;

use version; $VERSION = qv('0.0.2');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Perl6::Export::Attrs;

use POWER::Utils;
use POWER::Event::IO    qw( :event );
use POWER::Event::Timer qw( :event );

my @Events;

sub loop :Export(:DEFAULT) {
    while (1) {
        while (@Events) {
            my $event = shift @Events;
            next if !defined $event->[0]; # object was destroyed, skip event...
            run_callback(@$event);
        }
        my $idle = get_timers_idle_time();
        if (defined $idle && !$idle) {
            @Events = get_expired_timers();
        } else {
            (my $status, @Events) 
                = io_wait(defined $idle ? $idle+0.000999999 : undef);
        }
    }
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Event - event loop


=head1 VERSION

This document describes POWER::Event version 0.0.2


=head1 SYNOPSIS

 use POWER::Event;

 # activate event generators:
 # 
 #   use POWER::Event::Timer;
 #   set_timer({ ... });
 # 
 #   use POWER::Event::Epoll;
 #   Epoll_add(...);

 loop();
 die "exception in loop: $@\n";
 # OR with handling exceptions
 while (1) {
     eval { loop() };
     # process exception in $@
 }


=head1 DESCRIPTION

Main 'event loop' for user's code.

Current version compatible only with event generated by
POWER::Event::Timer and POWER::Event::Epoll.
Other types of events (signal, at some time, user-generated) may be
added in future versions, if needed.


=head1 EXPORTS

=over

=item :DEFAULT

loop()


=back

=head1 INTERFACE 

=over

=item loop( )

Will poll POWER::Event::Timer and POWER::Event::Epoll for events in endless
loop and execute events they generate by POWER::Utils/run_callback() .

Return: nothing. Only way to get out from loop() - if some exception will be
throws by event generators/handlers.


=back

=head1 DIAGNOSTICS

No exceptions or warnings generated. But all exceptions in event generators
or handlers will be propagated out of loop(), so from user's perspective
all of them looks like generated by loop().


=head1 CONFIGURATION AND ENVIRONMENT

POWER::Event requires no configuration files or environment variables.


=head1 DEPENDENCIES

=over

=item version

=item Perl6::Export::Attrs

=item POWER::Utils

=item POWER::Event::Timer

=item POWER::Event::IO


=back

=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-event@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
